@@ -102,73 +102,83 @@ KNOWN_COUNTRIES = {
     "ITA": "ITA",
 }
 
-TEST_CHANNEL_LIST = {
-    "documentId": "2416",
-    "services": [
-        {
-            "c": "317",
-            "dvbtriplet": "64511.8800.11684",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 12,
-            "sid": "684",
-            "sk": 684,
-            "t": "Premium Comedy HD",
-            "xsg": 3,
-        },
-        {
-            "c": "105",
-            "dvbtriplet": "64511.6400.11075",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 13,
-            "sid": "5435",
-            "sk": 75,
-            "t": "Canale 5 HD",
-            "xsg": 4,
-        },
-        {
-            "c": "120",
-            "dvbtriplet": "64511.6400.11074",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 21,
-            "sid": "74",
-            "sk": 74,
-            "t": "Sky Arte HD",
-            "xsg": 1,
-        },
-        {
-            "c": "999",
-            "dvbtriplet": "64511.6400.11074",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "hd",
-            "sg": 21,
-            "sid": "99",
-            "sk": 99,
-            "t": "Fake Channel",
-            "xsg": 1,
-        },
-        {
-            "c": "101",
-            "dvbtriplet": "2.2048.10353",
-            "schedule": True,
-            "servicetype": "DSAT",
-            "servicetypes": ["DSAT"],
-            "sf": "sd",
-            "sg": 12,
-            "sid": "2153",
-            "sk": 2153,
-            "t": "BBC One South",
-            "xsg": 3,
-        },
-    ],
-}
+TEST_CHANNEL_LIST = [
+    {
+        "c": "317",
+        "dvbtriplet": "64511.8800.11684",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "hd",
+        "sg": 12,
+        "sid": "684",
+        "sk": 684,
+        "t": "Premium Comedy HD",
+        "xsg": 3,
+    },
+    {
+        "c": "105",
+        "dvbtriplet": "64511.6400.11075",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "hd",
+        "sg": 13,
+        "sid": "5435",
+        "sk": 75,
+        "t": "Canale 5 HD",
+        "xsg": 4,
+    },
+    {
+        "c": "303",
+        "dvbtriplet": "64511.6400.11074",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "hd",
+        "sg": 21,
+        "sid": "1234",
+        "sk": 1234,
+        "t": "Sky Cinema Sci-Fi HD",
+        "xsg": 1,
+    },
+    {
+        "c": "120",
+        "dvbtriplet": "64511.6400.11074",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "hd",
+        "sg": 21,
+        "sid": "74",
+        "sk": 74,
+        "t": "Sky Arte HD",
+        "xsg": 1,
+    },
+    {
+        "c": "999",
+        "dvbtriplet": "64511.6400.11074",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "hd",
+        "sg": 21,
+        "sid": "99",
+        "sk": 99,
+        "t": "Fake Channel",
+        "xsg": 1,
+    },
+    {
+        "c": "101",
+        "dvbtriplet": "2.2048.10353",
+        "schedule": True,
+        "servicetype": "DSAT",
+        "servicetypes": ["DSAT"],
+        "sf": "sd",
+        "sg": 12,
+        "sid": "2153",
+        "sk": 2153,
+        "t": "BBC One South",
+        "xsg": 3,
+    },
+]
