@@ -21,6 +21,13 @@ class SkyQCountry:
 
     def getEpgData(self, sid, channelno, epgDate):
         """Get EPG data for UK."""
+        return self._getData(sid, channelno, epgDate)
+
+    def getTimeEpgData(self, sid, channelno, queryDateFrom, queryDateTo):
+        """Get EPG data for UK."""
+        return self._getData(sid, channelno, queryDateFrom)
+
+    def _getData(self, sid, channelno, epgDate):
         epgDateStr = epgDate.strftime("%Y%m%d")
 
         epgUrl = SCHEDULE_URL.format(sid, epgDateStr)
