@@ -4,7 +4,7 @@ import logging
 import requests
 
 from ..const import RESPONSE_OK
-from ..programme import Programme
+from ..classes.programme import Programme
 
 from .const_it import (
     CHANNEL_IMAGE_URL,
@@ -20,13 +20,11 @@ _LOGGER = logging.getLogger(__name__)
 class SkyQCountry:
     """Italy specific SkyQ."""
 
-    def __init__(self, host):
+    def __init__(self):
         """Initialise Italy remote."""
         self.channel_image_url = CHANNEL_IMAGE_URL
         self.pvr_image_url = PVR_IMAGE_URL
-        self._host = host
         self._channellist = None
-        self.country = "ITA"
 
         self._getChannels()
 
@@ -34,7 +32,15 @@ class SkyQCountry:
         """Get EPG data for Italy."""
         queryDateFrom = epgDate.strftime("%Y-%m-%dT00:00:00Z")
         queryDateTo = epgDate.strftime("%Y-%m-%dT23:59:59Z")
+        return self._getData(sid, channelno, queryDateFrom, queryDateTo)
 
+    def getTimeEpgData(self, sid, channelno, queryDateFrom, queryDateTo):
+        """Get EPG data for Italy."""
+        queryDateFrom = queryDateFrom.strftime("%Y-%m-%dT%H:%M:%SZ")
+        queryDateTo = queryDateTo.strftime("%Y-%m-%dT%H:%M:%SZ")
+        return self._getData(sid, channelno, queryDateFrom, queryDateTo)
+
+    def _getData(self, sid, channelno, queryDateFrom, queryDateTo):
         cid = None
         for channel in self._channellist:
             if str(channel["number"]) == str(channelno):
