@@ -10,6 +10,7 @@ import importlib
 import pycountry
 from datetime import datetime, timedelta
 from http import HTTPStatus
+from operator import attrgetter
 
 import websocket
 
@@ -49,10 +50,12 @@ from .const import (
     KNOWN_COUNTRIES,
 )
 from .const import TEST_CHANNEL_LIST
-from .channel import Channel
-from .programme import RecordedProgramme
-from .media import Media
-from .device import Device
+from .classes.channelepg import ChannelEPG
+from .classes.channellist import ChannelList
+from .classes.channel import Channel
+from .classes.programme import RecordedProgramme
+from .classes.media import Media
+from .classes.device import Device
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -62,21 +65,23 @@ class SkyQRemote:
 
     commands = COMMANDS
 
-    def __init__(
-        self, host, overrideCountry=None, test_channel=None, port=49160, jsonport=9006,
-    ):
+    def __init__(self, host, port=49160, jsonPort=9006):
         """Stand up a new SkyQ box."""
+        self.deviceSetup = False
         self._host = host
+        self._remoteCountry = None
+        self._overrideCountry = None
+        self._epgCountryCode = None
         self._serialNumber = None
         self._channel = None
-        self._test_channel = test_channel
+        self._test_channel = None
         self._port = port
-        self._jsonport = jsonport
-        self._overrideCountry = overrideCountry
+        self._jsonport = jsonPort
         self._soapControlURL = None
         self._lastEpg = None
         self._currentApp = APP_EPG
-        self.deviceSetup = False
+        self._channels = []
+        self._error = False
 
         self._setupDevice()
 
@@ -85,10 +90,14 @@ class SkyQRemote:
         if not self.deviceSetup:
             self._setupDevice()
 
+        if not self._remoteCountry:
+            SkyQCountry = self._importCountry(self._epgCountryCode)
+            self._remoteCountry = SkyQCountry()
+
         if self._soapControlURL is None:
             return SKY_STATE_OFF
 
-        output = self._retrieveSystemInformation()
+        output = self._retrieveInformation(REST_PATH_SYSTEMINFO)
         if output is None:
             return SKY_STATE_OFF
         if "activeStandby" in output and output["activeStandby"] is True:
@@ -126,14 +135,6 @@ class SkyQRemote:
         except Exception:
             return self._currentApp
 
-    def getCurrentMediaJSON(self):
-        """Get the currently playing media on the SkyQ box as json."""
-        media = self.getCurrentMedia()
-        if not media:
-            return None
-
-        return media.as_json()
-
     def getCurrentMedia(self):
         """Get the currently playing media on the SkyQ box."""
         channel = None
@@ -154,8 +155,10 @@ class SkyQRemote:
                         sid = self._test_channel
 
                     live = True
-                    channel = self._getChannelNode(sid)["channel"]
-                    imageUrl = self._buildChannelUrl(sid, channel)
+                    channelNode = self._getChannelNode(sid)
+                    if channelNode:
+                        channel = channelNode["channel"]
+                        imageUrl = self._buildChannelUrl(sid, channel)
                 elif PVR in currentURI:
                     # Recorded content
                     pvrId = "P" + currentURI[11:]
@@ -164,59 +167,66 @@ class SkyQRemote:
 
         return media
 
-    def getEpgDataJSON(self, sid, epgDate):
-        """Get EPG data for the specified channel as json."""
-        channel = self.getEpgData(sid, epgDate)
-        if not channel:
-            return None
-
-        return channel.as_json()
-
-    def getEpgData(self, sid, epgDate, days=2):
+    def getEpgData(self, sid, epgDate, days=2, timeSpecific=False):
         """Get EPG data for the specified channel/date."""
-        channelNode = self._getChannelNode(sid)
-        channelno = channelNode["channelno"]
-        channelname = channelNode["channel"]
-        channelImageUrl = self._buildChannelUrl(sid, channelname)
-
         epg = str(sid) + epgDate.strftime("%Y%m%d")
-        if self._lastEpg == epg:
+        if self._lastEpg == epg and not timeSpecific:
             return self._channel
-
-        programmes = set()
-        for n in range(days):
-            programmesData = self._remoteCountry.getEpgData(
-                sid, channelno, epgDate + timedelta(days=n)
-            )
-            if len(programmesData) > 0:
-                programmes = programmes.union(programmesData)
-            else:
-                break
-
         self._lastEpg = epg
 
-        if len(programmes) == 0:
-            _LOGGER.info(f"I0020 - Programme data not found for {sid} : {epgDate}")
+        channelNo = None
+        channelName = None
+        channelImageUrl = None
+        programmes = set()
 
-        self._channel = Channel(
-            sid, channelno, channelname, channelImageUrl, sorted(programmes)
+        channelNode = self._getChannelNode(sid)
+        if channelNode:
+            channelNo = channelNode["channelno"]
+            channelName = channelNode["channel"]
+            channelImageUrl = self._buildChannelUrl(sid, channelName)
+
+            if not timeSpecific:
+                for n in range(days):
+                    programmesData = self._remoteCountry.getEpgData(
+                        sid, channelNo, epgDate + timedelta(days=n)
+                    )
+                    if len(programmesData) > 0:
+                        programmes = programmes.union(programmesData)
+                    else:
+                        break
+            else:
+                programmes = self._remoteCountry.getTimeEpgData(
+                    sid, channelNo, epgDate, epgDate + timedelta(hours=12)
+                )
+
+        self._channel = ChannelEPG(
+            sid, channelNo, channelName, channelImageUrl, sorted(programmes)
         )
 
         return self._channel
 
-    def getProgrammeFromEpgJSON(self, sid, epgDate, queryDate):
-        """Get programme from EPG for specfied time and channel as json."""
-        programme = self.getProgrammeFromEpg(sid, epgDate, queryDate)
-        if not programme:
-            return None
-
-        return programme.as_json()
-
     def getProgrammeFromEpg(self, sid, epgDate, queryDate):
         """Get programme from EPG for specfied time and channel."""
         epgData = self.getEpgData(sid, epgDate)
+
+        timechange = 0
+        while (
+            len(epgData.programmes) > 0 and queryDate < epgData.programmes[0].starttime
+        ):
+            timechange += 1
+            epgData = self.getEpgData(
+                sid, epgDate - timedelta(hours=timechange), timeSpecific=True
+            )
+
         if len(epgData.programmes) == 0:
-            return None
+            if not self._error:
+                self._error = True
+                _LOGGER.info(
+                    f"I0020 - Programme data not found for SID {sid} : {epgDate}"
+                )
+                return PAST_END_OF_EPG
+        else:
+            self._error = False
 
         try:
             programme = next(
@@ -229,34 +239,21 @@ class SkyQRemote:
         except StopIteration:
             return PAST_END_OF_EPG
 
-    def getCurrentLiveTVProgrammeJSON(self, sid):
-        """Get current live programme on the specified channel as json."""
-        programme = self.getCurrentLiveTVProgramme(sid)
-        if not programme:
-            return None
-
-        return programme.as_json()
-
     def getCurrentLiveTVProgramme(self, sid):
         """Get current live programme on the specified channel."""
         try:
             queryDate = datetime.utcnow()
+            # queryDate = datetime.strptime(
+            #     "2020-06-05 00:15:27.243860", "%Y-%m-%d %H:%M:%S.%f"
+            # )
             programme = self.getProgrammeFromEpg(sid, queryDate, queryDate)
             if programme is None:
-                return None
+                return PAST_END_OF_EPG
 
             return programme
         except Exception as err:
             _LOGGER.exception(f"X0030 - Error occurred: {self._host} : {sid} : {err}")
-            return None
-
-    def getRecordingJSON(self, pvrId):
-        """Get the recording details as json."""
-        recording = self.getRecording(pvrId)
-        if not recording:
-            return None
-
-        return recording.as_json()
+            return PAST_END_OF_EPG
 
     def getRecording(self, pvrId):
         """Get the recording details."""
@@ -286,7 +283,7 @@ class SkyQRemote:
             imageUrl = self._remoteCountry.pvr_image_url.format(str(programmeuuid))
         elif "osid" in recording:
             sid = str(recording["osid"])
-            imageUrl = self._buildChannelUrl(sid)
+            imageUrl = self._buildChannelUrl(sid, channel)
 
         starttime = datetime.utcfromtimestamp(recording["ast"])
         if "finald" in recording:
@@ -302,57 +299,72 @@ class SkyQRemote:
 
         return programme
 
-    def getDeviceInformationJSON(self):
-        """Get the device information from the SkyQ box as json."""
-        device = self.getDeviceInformation()
-        if not device:
-            return None
-
-        return device.as_json()
-
     def getDeviceInformation(self):
         """Get the device information from the SkyQ box."""
-        device = None
-        deviceInfo = self._retrieveDeviceInformation()
+        deviceInfo = self._retrieveInformation(REST_PATH_DEVICEINFO)
+        if not deviceInfo:
+            return None
 
-        if deviceInfo:
-            systemInfo = self._retrieveSystemInformation()
-            ASVersion = deviceInfo["ASVersion"]
-            IPAddress = deviceInfo["IPAddress"]
-            countryCode = deviceInfo["countryCode"]
-            hardwareModel = systemInfo["hardwareModel"]
-            hardwareName = deviceInfo["hardwareName"]
-            manufacturer = systemInfo["manufacturer"]
-            modelNumber = deviceInfo["modelNumber"]
-            serialNumber = deviceInfo["serialNumber"]
-            versionNumber = deviceInfo["versionNumber"]
+        systemInfo = self._retrieveInformation(REST_PATH_SYSTEMINFO)
+        ASVersion = deviceInfo["ASVersion"]
+        IPAddress = deviceInfo["IPAddress"]
+        countryCode = deviceInfo["countryCode"]
+        hardwareModel = systemInfo["hardwareModel"]
+        hardwareName = deviceInfo["hardwareName"]
+        manufacturer = systemInfo["manufacturer"]
+        modelNumber = deviceInfo["modelNumber"]
+        serialNumber = deviceInfo["serialNumber"]
+        versionNumber = deviceInfo["versionNumber"]
 
-            epgCountryCode = None
-            if self._overrideCountry:
-                epgCountryCode = self._overrideCountry
-            else:
-                epgCountryCode = countryCode.upper()
-            if not epgCountryCode:
-                _LOGGER.error(f"E0050 - No country identified: {self._host}")
-                return None
+        if self._overrideCountry:
+            epgCountryCode = self._overrideCountry
+        else:
+            epgCountryCode = countryCode.upper()
+        if not epgCountryCode:
+            _LOGGER.error(f"E0050 - No country identified: {self._host}")
+            return None
 
-            if epgCountryCode in KNOWN_COUNTRIES:
-                epgCountryCode = KNOWN_COUNTRIES[epgCountryCode]
+        if epgCountryCode in KNOWN_COUNTRIES:
+            epgCountryCode = KNOWN_COUNTRIES[epgCountryCode]
 
-            device = Device(
-                ASVersion,
-                IPAddress,
-                countryCode,
-                epgCountryCode,
-                hardwareModel,
-                hardwareName,
-                manufacturer,
-                modelNumber,
-                serialNumber,
-                versionNumber,
-            )
+        self._epgCountryCode = epgCountryCode
+
+        device = Device(
+            ASVersion,
+            IPAddress,
+            countryCode,
+            epgCountryCode,
+            hardwareModel,
+            hardwareName,
+            manufacturer,
+            modelNumber,
+            serialNumber,
+            versionNumber,
+        )
         return device
 
+    def getChannelList(self):
+        """Get Channel list for Sky Q box."""
+        channels = self._getChannels()
+        if not channels:
+            return None
+
+        channelitems = set()
+
+        for c in channels:
+            channelno = c["c"]
+            channelname = c["t"]
+            sf = c["sf"]
+            channel = Channel(channelno, channelname, sf=sf)
+            channelitems.add(channel)
+
+        channelnosorted = sorted(channelitems, key=attrgetter("channelno"))
+        self._channellist = ChannelList(
+            sorted(channelnosorted, key=attrgetter("channeltype"), reverse=True)
+        )
+
+        return self._channellist
+
     def press(self, sequence):
         """Issue the specified sequence of commands to SkyQ box."""
         if isinstance(sequence, list):
@@ -368,6 +380,19 @@ class SkyQRemote:
             else:
                 self._sendCommand(self.commands[sequence.casefold()])
 
+    def setOverrides(
+        self, overrideCountry=None, test_channel=None, jsonPort=None, port=None
+    ):
+        """Override various items."""
+        if overrideCountry:
+            self._overrideCountry = overrideCountry
+        if test_channel:
+            self._test_channel = test_channel
+        if jsonPort:
+            self._jsonport = jsonPort
+        if port:
+            self.port = port
+
     def _http_json(self, path, headers=None) -> str:
         response = requests.get(
             REST_BASE_URL.format(self._host, self._jsonport, path),
@@ -494,9 +519,7 @@ class SkyQRemote:
 
             if time.time() > timeout:
                 _LOGGER.error(
-                    "E0030 - Timeout error sending command: {self._host} : {0}".format(
-                        str(code)
-                    )
+                    f"E0030 - Timeout error sending command: {self._host} : {str(code)}"
                 )
                 break
 
@@ -506,13 +529,33 @@ class SkyQRemote:
         return channel_image_url.format(sid, chid)
 
     def _getChannelNode(self, sid):
-        channelNode = next(
-            s for s in self._channels["services"] if s["sid"] == str(sid)
-        )
+        channelNode = self._getNodeFromChannels(sid)
+
+        if not channelNode:
+            # Load the channel list for the first time.
+            # It's also possible the channels may have changed since last HA restart, so reload them
+            self._channels = self._getChannels()
+            channelNode = self._getNodeFromChannels(sid)
+            if not channelNode:
+                return None
+
         channel = channelNode["t"]
         channelno = channelNode["c"]
         return {"channel": channel, "channelno": channelno}
 
+    def _getChannels(self):
+        # This is here because otherwise I can never validate code for a foreign device
+        if self._test_channel:
+            return TEST_CHANNEL_LIST
+        channels = self._http_json(REST_CHANNEL_LIST)
+        if channels:
+            return channels["services"]
+
+        return []
+
+    def _getNodeFromChannels(self, sid):
+        return next((s for s in self._channels if s["sid"] == str(sid)), None)
+
     def _setupDevice(self):
         """Set the remote up."""
         deviceInfo = self.getDeviceInformation()
@@ -525,36 +568,13 @@ class SkyQRemote:
             self._soapControlURL = self._getSoapControlURL(url_index)["url"]
             url_index += 1
 
-        SkyQCountry = self._importCountry(deviceInfo)
-
-        self._remoteCountry = SkyQCountry(self._host)
-
-        # This is here because otherwise I can never validate code for a foreign device
-        if not self._test_channel:
-            self._channels = self._http_json(REST_CHANNEL_LIST)
-        else:
-            self._channels = TEST_CHANNEL_LIST
-
         self.deviceSetup = True
 
         return
 
-    def _retrieveDeviceInformation(self):
+    def _retrieveInformation(self, rest_path):
         try:
-            resp = self._http_json(REST_PATH_DEVICEINFO)
-            return resp
-        except (
-            requests.exceptions.ConnectTimeout,
-            requests.exceptions.ConnectionError,
-        ):
-            return None
-        except Exception as err:
-            _LOGGER.exception(f"X0080 - Error occurred: {self._host} : {err}")
-            return None
-
-    def _retrieveSystemInformation(self):
-        try:
-            resp = self._http_json(REST_PATH_SYSTEMINFO)
+            resp = self._http_json(rest_path)
             return resp
         except (
             requests.exceptions.ConnectTimeout,
@@ -563,21 +583,19 @@ class SkyQRemote:
         ):
             return None
         except Exception as err:
-            _LOGGER.exception(f"X0090 - Error occurred: {self._host} : {err}")
+            _LOGGER.exception(f"X0080 - Error occurred: {self._host} : {err}")
             return None
 
-    def _importCountry(self, deviceInfo):
+    def _importCountry(self, epgCountryCode):
         try:
-            country = pycountry.countries.get(
-                alpha_3=deviceInfo.epgCountryCode
-            ).alpha_2.casefold()
+            country = pycountry.countries.get(alpha_3=epgCountryCode).alpha_2.casefold()
             SkyQCountry = importlib.import_module(
                 "pyskyqremote.country.remote_" + country
             ).SkyQCountry
 
         except (AttributeError, ModuleNotFoundError) as err:
             _LOGGER.warning(
-                f"W0030 - Invalid country, defaulting to GBR : {self._host} : {deviceInfo.epgCountryCode} : {err}"
+                f"W0030 - Invalid country, defaulting to GBR : {self._host} : {epgCountryCode} : {err}"
             )
 
             from pyskyqremote.country.remote_gb import SkyQCountry
